==> m4e_sdk-0.0.3-dev.11.tar.gz: lib/src/data/api/permission_api.dart <==
import 'dart:convert';
import 'dart:io';

import 'package:dio/dio.dart';
import 'package:flutter/foundation.dart';

import '../../core/core.dart';
import '../../m4e_forms/m4e_forms.dart';

/// Accessing the instantiated `PermissionApi` methods
/// is done by passing its type to the GetIt instance
///
/// Example use of `AuthApi`:
/// ```
/// import 'package:m4edart/m4e_sdk_initialize.dart' as sdk;
///
/// sdk.m4e<AuthApi>().getCurrentUser();
/// ```
///
abstract class PermissionApi {
  /// Sets permissions of a subject on an object
  ///
  /// Returns [UniqueId] of object set permission on
  ///
  /// Throws [ServerException] if call to server fails
  ///
  /// Throws [ServerException] with `NO_INTERNET_CONNECTION` error
  /// code device is not connected to the internet
  ///
  Future<VoidImpl> setPermission(M4ePermissionForm permissionForm);
}

class PermissionApiImpl implements PermissionApi {
  final Dio _client;

  PermissionApiImpl({
    @required Dio client,
    @required M4eLoggingInterceptors loggingInterceptor,
    @required M4eHeaderInterceptors headerInterceptor,
  })  : assert(client != null),
        assert(loggingInterceptor != null),
        assert(headerInterceptor != null),
        _client = client {
    _client?.interceptors?.add(loggingInterceptor);
    _client?.interceptors?.add(headerInterceptor);
  }

  @override
  Future<VoidImpl> setPermission(M4ePermissionForm permissionForm) async {
    try {
      final _response = await _client.put('/permissions',
          data: json.encode(permissionForm.toJson()));

      if (_response.statusCode == 200) {
        return VoidImpl();
      }

      throw M4eExceptionMessages.kFailedTransactionException;
    } on DioError catch (e) {
      // print error in console in debug mode
      logger(e);

      if (e.type == DioErrorType.CONNECT_TIMEOUT) {
        throw M4eExceptionMessages.kConnectionTimeOutException;
      }
      if (e.type == DioErrorType.RESPONSE) {
        if (e.response.statusCode == 401) {
          // TODO: implement mapping and returning error responses from server
        }
        if (e.response.statusCode == 404) {
          throw M4eExceptionMessages.kNoNotFoundException;
        }
      }

      throw M4eExceptionMessages.kUndefinedException;
    } on SocketException {
      throw M4eExceptionMessages.kNoInternetConnectionException;
    } catch (e) {
      throw M4eExceptionMessages.kUndefinedException;
    }
  }
}

==> m4e_sdk-0.0.4-dev.10.tar.gz: lib/src/data/api/permission_api.dart <==
import 'dart:convert';
import 'dart:io';

import 'package:dio/dio.dart';
import 'package:flutter/foundation.dart';

import '../../core/core.dart';
import '../../m4e_forms/m4e_forms.dart';

/// Accessing the instantiated `PermissionApi` methods
/// is done by passing its type to the GetIt instance
///
/// Example use of `AuthApi`:
/// ```
/// import 'package:m4edart/m4e_sdk_initialize.dart' as sdk;
///
/// sdk.m4e<AuthApi>().getCurrentUser();
/// ```
///
abstract class PermissionApi {
  /// Sets permissions of a subject on an object
  ///
  /// Returns [UniqueId] of object set permission on
  ///
  /// Throws [ServerException] if call to server fails
  ///
  /// Throws [ServerException] with `NO_INTERNET_CONNECTION` error
  /// code device is not connected to the internet
  ///
  Future<M4eUnitImpl> setPermission(M4ePermissionForm permissionForm);
}

class PermissionApiImpl implements PermissionApi {
  final Dio _client;

  PermissionApiImpl({
    @required Dio client,
    @required M4eLoggingInterceptors loggingInterceptor,
    @required M4eHeaderInterceptors headerInterceptor,
  })  : assert(client != null),
        assert(loggingInterceptor != null),
        assert(headerInterceptor != null),
        _client = client {
    _client?.interceptors?.add(loggingInterceptor);
    _client?.interceptors?.add(headerInterceptor);
  }

  @override
  Future<M4eUnitImpl> setPermission(M4ePermissionForm permissionForm) async {
    try {
      await _client.put('/permissions',
          data: json.encode(permissionForm.toJson()));

      return unitImpl;
    } on DioError catch (e) {
      if (e.type == DioErrorType.CONNECT_TIMEOUT) {
        throw M4eExceptionMessages.kConnectionTimeOutException;
      }

      if (e.response?.statusCode == 403) {
        throw M4eExceptionMessages.kUnauthorized;
      }

      if (e.response?.statusCode == 503) {
        throw M4eExceptionMessages.kServiceUnavailableException;
      }

      if (e.type == DioErrorType.RESPONSE) {
        throw M4eServerException.fromJson(e.response.data);
      }

      throw M4eServerException.fromString(e.message.toString());
    } on SocketException {
      throw M4eExceptionMessages.kNoInternetConnectionException;
    } on HttpException {
      throw M4eExceptionMessages.kHttpException;
    } on FormatException {
      throw M4eExceptionMessages.kHttpFormatException;
    } catch (e) {
      throw M4eServerException.fromString(e.toString());
    }
  }
}
